//analogpots.cpp

#include "afgcockpit.h"
#include "analogpots.h"

#if NANALOGPOTS > 0

void setupAnalogPots() {
  
  for (int a=0; a<NANALOGPOTS; a++) {
    analogpots[a].pin = analogpots_pins[a];
    #ifdef ARDEBUG
    Serial.print("Analog pot on pin ");
    Serial.print(analogpots[a].pin);
    Serial.print(" mode=");
    Serial.print(analogpots[a].mode);
    Serial.print(" precision=");
    Serial.print(analogpots[a].precision);
    Serial.print(" accuracy=");
    Serial.println(analogpots[a].accuracy);
    #endif
  }
}
void readAnalogPots() {
  for (int a=0; a<NANALOGPOTS; a++)
    readAnalogPot(&analogpots[a]);
}

void readAnalogPot(AnalogPot *pot) {
  float tmpmap = 0;
  int analogpin = pot->pin;

  if (pot->timer + pot->delay > millis())
    return;
  pot->timer = millis();

  float tmpraw=0;
  for (int i=0; i<pot->measure_averages; i++) {
    delayMicroseconds(pot->measure_delay);
    tmpraw += analogRead(analogpin);
  }
  tmpraw /= pot->measure_averages;

  if (!OutsideRange(tmpraw, pot->raw_value, pot->accuracy))
    return;

  tmpmap = mapf(tmpraw, pot->minin, pot->maxin, pot->minout, pot->maxout);

    if (pot->mode == ABSOLUTE_VALUE) {
      pot->value = tmpmap;        // Update the value
    }
    else if (pot->mode == RELATIVE_VALUE) {
      pot->value = pot->value + tmpmap - mapf(pot->raw_value, pot->minin, pot->maxin, pot->minout, pot->maxout);
    }
    else if (pot->mode == MIMIC_PUSH_BUTTON) {
      // we shall emulate a sticky push button.

      if (pot->value == LOW && tmpmap == HIGH) {
        pot->value = HIGH;
      }
      else if (pot->value == HIGH && tmpmap == HIGH) {
        pot->value = LOW;
      }
    }
    else {
      Serial.println("Ooops! Bad type for muxer.");
    }

  pot->raw_value = tmpraw;
  sendmsg = true;
}

void printAnalogPots() {
  for (int i=0; i<NANALOGPOTS; i++) {
    PrintDouble(analogpots[i].value, analogpots[i].precision);
    Serial.print(",");
  }
}

void clearAnalogPots() {
  // If a pot is in mode=relative we will clear this value, otherwise
  // it will keep on sending same value all the time.
  for (int i=0; i<NANALOGPOTS; i++) {
    if (analogpots[i].mode == RELATIVE_VALUE)
      analogpots[i].value = 0;
  }
}

#endif
