// mux.cpp

#include "afgcockpit.h"
#include "mux.h"


#if NMUXERCONTROLLERS > 0

struct MuxController {
  // Control pins for the mux.
  // Multiple muxers share same pins, only
  // analog input will differ. At least that seems to be
  // best to reduce number of used pins, so that is what
  // is assumed here and in code below.
  const int pin_s0 = MUX_PIN_S0;
  const int pin_s1 = MUX_PIN_S1;
  const int pin_s2 = MUX_PIN_S2;
  int pin_analogin = 0;    // analog input pin
} muxcontroller[NMUXERCONTROLLERS];


void setupMuxers() {
  #ifdef ARDEBUG
  Serial.println("setup muxers");
  #endif

  pinMode(muxcontroller[0].pin_s0, OUTPUT);    // mux_s0
  pinMode(muxcontroller[0].pin_s1, OUTPUT);    // mux_s1
  pinMode(muxcontroller[0].pin_s2, OUTPUT);    // mux_s2

  for (int i=0; i<NMUXERCONTROLLERS; i++) {
    muxcontroller[i].pin_analogin = muxer_analogin_pins[i];
  }

  for (int i=0; i<NMUXERCHANNELS; i++) {
    // Info about all muxer channels are stored in pairs: muxer,channel
    mux[i].mux_controller = muxer_address[i*2];
    mux[i].mux_channel = muxer_address[i*2+1];
  }

}

void readMuxers() {
  for (int i=0; i<NMUXERCHANNELS; i++)
    readMuxer(&mux[i]);
}

void readMuxer(MuxAnalog *pot) {
  float tmpmap = 0;
  int mux_r0 = 0;      //value select pin at the 4051 (mux_s0)
  int mux_r1 = 0;      //value select pin at the 4051 (mux_s1)
  int mux_r2 = 0;      //value select pin at the 4051 (mux_s2)
  int mux_row = 0;     // storeing the bin code
  int mux_bin [8] = {000, 1, 10, 11, 100, 101, 110, 111};//bin = binär, some times it is so easy

  int ctrl = pot->mux_controller;
  int channel = pot->mux_channel;
  int analogpin = muxcontroller[ctrl].pin_analogin;

  if (pot->timer + pot->delay > millis())
    return;
  pot->timer = millis();

  mux_row = mux_bin[channel];
  mux_r0 = mux_row & 0x01;
  mux_r1 = (mux_row>>1) & 0x01;
  mux_r2 = (mux_row>>2) & 0x01;
  
  digitalWrite(muxcontroller[0].pin_s0, mux_r0);
  digitalWrite(muxcontroller[0].pin_s1, mux_r1);
  digitalWrite(muxcontroller[0].pin_s2, mux_r2);

  float tmpraw=0;
  for (int i=0; i<pot->measure_averages; i++) {
    delayMicroseconds(pot->measure_delay);
    tmpraw += analogRead(analogpin);
  }
  tmpraw /= pot->measure_averages;

  if (!OutsideRange(tmpraw, pot->raw_value, pot->accuracy))
    return;

  tmpmap = mapf(tmpraw, pot->minin, pot->maxin, pot->minout, pot->maxout);

    if (pot->mode == ABSOLUTE_VALUE) {
      pot->value = tmpmap;        // Update the value
    }
    else if (pot->mode == RELATIVE_VALUE) {
      pot->value = pot->value + tmpmap - mapf(pot->raw_value, pot->minin, pot->maxin, pot->minout, pot->maxout);
    }
    else if (pot->mode == MIMIC_PUSH_BUTTON) {
      // we shall emulate a sticky push button.

      if (pot->value == LOW && tmpmap == HIGH) {
        pot->value = HIGH;
      }
      else if (pot->value == HIGH && tmpmap == HIGH) {
        pot->value = LOW;
      }
    }
    else {
      Serial.println("Ooops! Bad type for muxer.");
    }

  pot->raw_value = tmpraw;
  sendmsg = true;
}


void printMuxers() {
  for (int i=0; i<NMUXERCHANNELS; i++) {
    PrintDouble(mux[i].value, mux[i].precision);
    Serial.print(",");
  }
}

void clearMuxers() {
  // If a pot is in mode=relative we will clear this value, otherwise
  // it will keep on sending same value all the time.
  for (int i=0; i<NMUXERCHANNELS; i++) {
    if (mux[i].mode == RELATIVE_VALUE)
      mux[i].value = 0;
  }
}


#endif
